﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Text;

using DevExpress.XtraEditors;
using DevExpress.XtraRichEdit;
using DevExpress.XtraRichEdit.API.Native;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : XtraForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 줄 문자열
        /// </summary>
        private const string LINE_STRING = "------------------------------------------------------------------------------------------------------------------------";

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.richEditControl.ActiveViewType = RichEditViewType.Simple;

            this.richEditControl.Options.HorizontalScrollbar.Visibility = RichEditScrollbarVisibility.Hidden;
            this.richEditControl.Options.VerticalScrollbar.Visibility   = RichEditScrollbarVisibility.Auto;

            this.richEditControl.Document.DefaultCharacterProperties.FontName  = "나눔고딕코딩";
            this.richEditControl.Document.DefaultCharacterProperties.FontSize  = 12f;
            this.richEditControl.Document.DefaultCharacterProperties.ForeColor = Color.Black;

            this.richEditControl.Document.ParagraphStyles[0].FontName  = "나눔고딕코딩";
            this.richEditControl.Document.ParagraphStyles[0].FontSize  = 12f; 
            this.richEditControl.Document.ParagraphStyles[0].ForeColor = Color.Black;

            this.richEditControl.LoadDocument("sample.rtf");

            this.parseButton.Click += parseButton_Click;
        }

        #endregion


        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 분석 버튼 클릭시 처리하기 - parseButton_Click(sender, e)

        /// <summary>
        /// 분석 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void parseButton_Click(object sender, EventArgs e)
        {
            Document document = this.richEditControl.Document;

            Dictionary<Table, List<Paragraph>> tableDictionary = new Dictionary<Table, List<Paragraph>>();

            Dictionary<Paragraph, Table> paragraphDictionary = new Dictionary<Paragraph, Table>();

            foreach(Table table in document.Tables)
            {
                ParagraphCollection paragraphCollection = document.GetParagraphs(table.Range);

                List<Paragraph> paragraphList = new List<Paragraph>();

                for(int i = 0; i < paragraphCollection.Count; i++)
                {
                    Paragraph paragraph = paragraphCollection[i];

                    paragraphList.Add(paragraph);

                    paragraphDictionary.Add(paragraph, table);
                }

                tableDictionary.Add(table, paragraphList);
            }

            List<object> targetList = new List<object>();

            Dictionary<Table, string> targetTableDictionary = new Dictionary<Table, string>();

            foreach(Paragraph paragraph in document.Paragraphs)
            {
                ReadOnlyDocumentImageCollection imageCollection = document.Images.Get(paragraph.Range);

                if(imageCollection == null || imageCollection.Count == 0)
                {
                    if(paragraphDictionary.ContainsKey(paragraph))
                    {
                        Table table = paragraphDictionary[paragraph];

                        if(!targetTableDictionary.ContainsKey(table))
                        {
                            targetTableDictionary.Add(table, null);

                            targetList.Add(table);
                        }
                    }
                    else
                    {
                        targetList.Add(paragraph);
                    }
                }
                else
                {
                    DocumentImage documentImage = imageCollection[0];

                    targetList.Add(documentImage);
                }
            }

            StringBuilder stringBuilder = new StringBuilder();

            foreach(var item in targetList)
            {
                if(item is Paragraph paragraph)
                {
                    stringBuilder.AppendLine($"문단 : {document.GetText(paragraph.Range)}");
                }
                else if(item is DocumentImage documentImage)
                {
                    stringBuilder.AppendLine($"이미지 : {documentImage.Size}");
                }
                else if(item is Table table)
                {
                    stringBuilder.AppendLine($"테이블 : {table.Rows.Count}×{table.Rows[0].Cells.Count}");
                }
            }

            this.memoEdit.Text = stringBuilder.ToString();
        }

        #endregion
    }
}